#include <iostream>

#include "vtkConeSource.h"
#include "vtkPolyDataMapper.h"
#include "vtkRenderWindow.h"
#include "vtkLoopSubdivisionFilter.h"
#include "vtkTriangleFilter.h"
#include "vtkActorCollection.h"
#include "vtkPolyData.h"
#include "vtkRenderWindowInteractor.h"
#include "vtk3DSOurImporter.h"

int main( int argc, char *argv[] )
{
  if (argc != 3)
    {
      cout << "USAGE: " << argv[0] << " <filename.3DS> [triangle|quad]" << endl;
      exit(-1);
    }
  
  vtkRenderer *renderer = vtkRenderer::New();
  vtkRenderWindow *renderWindow = vtkRenderWindow::New();
  renderWindow->AddRenderer(renderer);
  renderer->SetBackground(0,0,0); 

  // read 3DS file
  // TODO: load file from program

  vtk3DSOurImporter *importer = vtk3DSOurImporter::New();
  importer->SetPolys(argv[2]);
  importer->ComputeNormalsOn();
  importer->SetFileName(argv[1]);
  importer->Read();
  
  vtkActorCollection *ac = importer->GetRenderer()->GetActors();
  ac->InitTraversal();
  vtkActor *dsActor=ac->GetNextActor();
  while (dsActor!=NULL) {
    vtkPolyData *polyData = (vtkPolyData*)(dsActor->GetMapper()->GetInputAsDataSet());

    // // mesh already triangulated by vtk3DSImporter?!?
    //
    // vtkTriangleFilter *triangleFilter = vtkTriangleFilter::New();
    // triangleFilter->SetInput(polyData);
    // 
    // // TODO: use subdivision class
    // // TODO: interactive subdivision level changes
            
    vtkPolyDataMapper *mapper = vtkPolyDataMapper::New();
    mapper->SetInput(polyData);
    vtkActor *actor = vtkActor::New();
    
    actor->SetMapper(mapper);
    actor->SetProperty(dsActor->GetProperty()); 
    renderer->AddActor(actor);
    dsActor=ac->GetNextActor();
  }
        
  vtkRenderWindowInteractor *interactor = vtkRenderWindowInteractor::New();
  interactor->SetRenderWindow(renderWindow);
        
  renderWindow->SetSize(500,500);
  renderWindow->Render();
  interactor->Start();
	
  ac->Delete();
  interactor->Delete();
  renderer->Delete();
  renderWindow->Delete();

  return 0;
}


